/* Hipervolume indicator */

#include <stdlib.h>
#include <stdio.h>

#include "emo.h"

int main(int argc, char **argv) {
  double *data, *ref, *v;
  int row, col, i, j, *filter, r, nrun;
  EMO_NDSort nd;
  EMO_HV hv;
  char *str;

  if(argc < 5) {
    printf("Syntax: %s prefix_Data_File #runs reference_point\n", argv[0]);
    exit(1);
  }

  str = (char *) malloc(sizeof(char) * 1000);

  nrun = atoi(argv[2]);
  r = 1;

  sprintf(str, "%s_R%.2d.pof", argv[1], r);
  printf("%s\n", str);

  row = col = 0;
  data = EMO_File_read(NULL, &row, &col, str, 0);
  printf("fil %d col %d \n", row, col);

  if(argc != ( 3 + col)) {
    printf("Missing arguments for the reference point.\n");
    printf("Syntax: %s prefix_Data_File #runs reference_point\n", argv[0]);
    exit(1);
  }

  ref = (double *) malloc(sizeof(double) * col);
  v = (double *) malloc(sizeof(double) * nrun);

  for(i = 0; i < col; i++) {
    ref[i] = atof(argv[3 + i]);
    printf("ref %d %f\n", i, ref[i]);
  }

  do { 
    filter = (int *) calloc(sizeof(int), row);
    EMO_NDSort_alloc(&nd, row);
    EMO_HV_alloc(&hv, row, col);

    // Filtrar soluciones no dominadas
    EMO_NDSort_run(&nd, data, NULL, row, col);

    //EMO_lprint(nd.front[0]);
    printf("nfront %d\n", nd.nfront);

    for(i = 0; i < nd.front[0].length; i++) {
      j = EMO_element(nd.front[0], i);
      filter[j] = 1;
    }

    // Fonseca
    v[r-1] = EMO_hv2(data, row, ref, col);
    //printf(" %f\n", v);

    // WFG
    //v[r-1] = EMO_HV_run(&hv, data, filter, row, ref);
    printf("%d %f\n", r-1, v[r-1]);

    if(++r <= nrun) {
      sprintf(str, "%s_R%.2d.pof", argv[1], r);
      printf("%s\n", str);
      free(data);
      row = 0;
      data = EMO_File_read(NULL, &row, &col, str,0);
    }

    free(filter);
    EMO_HV_free(&hv);
    EMO_NDSort_free(&nd);

  } while(r <= nrun);


  sprintf(str, "%s.hv", argv[1]);
  printf("Data stored in %s\n", str);
  EMO_File_write(v, NULL, nrun, 1, str, "%e ");

  free(str);
  free(data);
  free(ref);
  free(v);

  return 0;
}

